from setuptools import setup

setup( name='spycy'
     , python_requires='>=3'
     , version='0.2'
     , description='Small library for auto-currying functions and improved functional application'
     , url='http://github.com/kanales/spycy'
     , author='Ivan Canales'
     , author_email='kanales.contact@gmail.com'
     , license='MIT'
     , packages=['spycy']
     , zip_safe=False
     )
